// **********************************************************************
//  Segmentation algorithm class definitions
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef PICEXTSEGMALGS_HXX
#define PICEXTSEGMALGS_HXX

#include <PicSegmentAlgorithm.hxx>

/**
  * Quarter segmentation algorithm is not a useful segmentation algorithm,
  * but is just to give an example of a functionality needed. This algorithm
  * simply divides the image into four rectangular segments of equal size.
  * The segments produced have the following format:
  * \begin{itemize}
  *   \item 0..sizeof(PICUSHORT) -- left coordinate of the segment within
  *         the image.
  *   \item next sizeof(PICUSHORT) -- top coordinate.
  *   \item next sizeof(PICUSHORT) -- right coordinate.
  *   \item next sizeof(PICUSHORT) -- bottom coordinate.
  * \end{itemize}
  *
  * Only the coordinates are stored, not the actual image data within the
  * segment!
  * Algorithm is marked as isAutomatic == boolFalse.
  **/

class PicQuarterSegmentAlgorithm : public PicSegmentAlgorithm
{
 public:
  PICOBJECTCLASS(PicQuarterSegmentAlgorithm)

  /**
    * The default constructor.
    **/
  PicQuarterSegmentAlgorithm();

  /**
    * Export algorithm parameters. This algorithm has no parameters.
    *
    * @param params Collection object which is to receive the parameters.
    * @return True if successful.
    **/
  virtual PICBOOL setParams(PicIxCollect &params);
  /**
    * Import algorithm parameters. This algorithm has no parameters.
    *
    * @param params Collection object holding the parameters.
    * @return True if successful.
    **/
  virtual PICBOOL getParams(PicIxCollect &params);
  /**
    * Computes a PicSegmentation object from a specified picture object.
    * The user is responsible for disposing of the generated PicSegmentation
    * object.
    *
    * @param pic PicPicture object holding the picture data from which the
    *            segmentation object is to be computed.
    * @return Pointer to the created PicSegmentation object.
    **/
  virtual PicSegmentation* computeSegments(PicPicture &pic) const;
  /**
    * Returns a BaseString object of an Image object.
    *
    * @return BaseString object holding the image object data.
    **/
  virtual BaseString getImage() const;
};

#endif

