// **********************************************************************
//  Implementations of segmentation algorithms
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <palloc.hxx>
#include <picerror.hxx>
#include <PicExtSegmAlgs.hxx>

MAKE_PICOBJECTCLASS(PicQuarterSegmentAlgorithm, PicSegmentAlgorithm)

PicQuarterSegmentAlgorithm::PicQuarterSegmentAlgorithm()
{
  name = "Quarter";
  id = PIC_QUARTERSEGMALG_ID;
  isAutomatic = boolFalse;
}

PICBOOL PicQuarterSegmentAlgorithm::getParams(PicIxCollect &params)
{
  params.setNull();
  return boolTrue;
}

PICBOOL PicQuarterSegmentAlgorithm::setParams(PicIxCollect &params)
{
  return getParams(params);
}

BaseString PicQuarterSegmentAlgorithm::getImage() const
{
  return name;
}

static void segmSetInto(char *buf, 
                        PICUSHORT left, PICUSHORT top, 
                        PICUSHORT right, PICUSHORT bottom)
{
  char *cursor = buf;
  memcpy(cursor, &left, sizeof(left));
  cursor += sizeof(left);
  memcpy(cursor, &top, sizeof(top));
  cursor += sizeof(top);
  memcpy(cursor, &right, sizeof(right));
  cursor += sizeof(right);
  memcpy(cursor, &bottom, sizeof(bottom));
}

PicSegmentation* PicQuarterSegmentAlgorithm::computeSegments(PicPicture &pic) const
{
  PICUSHORT       left, right, top, bottom;
  PicSegment      seg;
  PicSegmentation *ret = new PicSegmentation;

  if(!ret || !pic.raw || pic.raw->isNull())
    return ret;

  if((seg.data = (char *)picAllocate(4*sizeof(PICUSHORT)))){
    seg.dataSize = 4*sizeof(PICUSHORT);

    ret->algId = id;

    left = 0; top = 0;
    right = pic.raw->getWidth() / 2; 
    bottom = pic.raw->getHeight() / 2;
    segmSetInto(seg.data, left, top, right, bottom);
    ret->segments.putAt(1, seg);

    left = right + 1; 
    right = pic.raw->getWidth(); 
    segmSetInto(seg.data, left, top, right, bottom);
    ret->segments.putAt(2, seg);

    left = 0; top = bottom + 1;
    right = pic.raw->getWidth() / 2; 
    bottom = pic.raw->getHeight();
    segmSetInto(seg.data, left, top, right, bottom);
    ret->segments.putAt(3, seg);

    left = right + 1; 
    right = pic.raw->getWidth(); 
    segmSetInto(seg.data, left, top, right, bottom);
    ret->segments.putAt(4, seg);
  }
  return ret;
}

