// **********************************************************************
//  This file contains PicImageFormat implementation
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <math.h>

#include <bstring.hxx>
#include <bstrarr.hxx>
#include <pfilestm.hxx>
#include <pstream.hxx>
#include <pinteger.hxx>
#include <picerror.hxx>
#include <ptext.hxx>

#include <PicImageFormat.hxx>

#include <mmain.hxx>

MAKE_PICABSOBJECTCLASS(PicImageFormat, PicObject)
MAKE_PICOBJECTCLASS(PicPPM, PicImageFormat)

PicImageFormat::PicImageFormat() :
  id(0), isLossy(boolFalse)
{
}

PicImageFormat::~PicImageFormat() 
{
}

PicObject& PicImageFormat::operator = (const PicObject& other)
{
  undefined("PicImageFormat::operator =");
  return *this;
}

PICBOOL PicImageFormat::operator == (const PicObject& other) const
{
  return (this == &other);
}

PICBOOL PicImageFormat::operator <  (const PicObject&) const
{
  undefined("PicImageFormat::operator <");
  return boolFalse;
}

PICBOOL PicImageFormat::isNull() const
{
  return(name.isEmpty());
}

void PicImageFormat::setNull()
{
  name.setNull();
}

BaseString PicImageFormat::getImage() const
{
  return name;
}

void PicImageFormat::setImage(const BaseString &image)
{
  undefined("PicImageFormat::setImage");
}

PicRawPicture *PicImageFormat::readPicture(BaseString fileName)
{
  PicErrorContext ecxt;
  PicFile	  file(fileName);
  PICULONG        fsize = file.itemCount();
  PicFileStream   fstr(fileName, PicFile::forReading);

  if(PicError::isDetected())
    return NULLPTR;

  return readPicture(fstr, fsize);
}

PICBOOL PicImageFormat::writePicture(BaseString fileName, PicRawPicture *raw)
{
  PicErrorContext ecxt;

  if(!raw || raw->isNull())
    return boolFalse;

  PicFile file(fileName);
  PicFileStream out(fileName, PicFile::forWriting);
  if(PicError::isDetected())
    return boolFalse;

  return writePicture(out, raw); 
}

//************************************************************

PicPPM::PicPPM()
{
  name = "PPM";
  id = PIC_PPM_IMAGEFORMAT_ID;
  fileExtensions.add("ppm");
  isLossy = boolFalse;
}

PicRawPicture *PicPPM::readPicture(PicStream &from, PICULONG size)
{
  char          magicValue[3];
  PICULONG      width, height, maxGrey;
  unsigned char *dataBuf;
  PicRawPicture *raw = NULLPTR;
  BaseString    val;
  PicInteger    intVal;
  char          forParse;
  PICSIZE       charsRead;

  PicText       text(from);

  do { 
    from.read(magicValue, 1);
    if (*magicValue=='#') {
      do{
        charsRead = from.read(magicValue, 1);
      } while (charsRead && *magicValue != '\n');
    }
    else break;
  }
  while (1);

  from.read(magicValue+1, 1);
  magicValue[2]='\0';

  if (strcmp(magicValue, "P6") && strcmp(magicValue, "P3")) {
    return NULL;
  }

  do { 
    from.read(&forParse, 1);
    if (forParse=='#') {
      do {
        charsRead = from.read(&forParse, 1);
      } while (charsRead && forParse != '\n');
    }
    else break;
  }
  while (1);

  do {
    text.rd(val, ' ');
    if (val.first()=='#') {
      do
        text.rd(forParse);
      while (forParse != '\n');
    }
    else {
      intVal.setImage(val);
      width=intVal.asLong();
      break;
    }
  }
  while (1);

  do {
    text.rd(val, ' ');
    if (val.first()=='#') {
      do
        text.rd(forParse);
      while (forParse != '\n');
    }
    else {
      intVal.setImage(val);
      height=intVal.asLong();
      break;
    }
  }
  while (1);

  do {
    text.rd(val, ' ');
    if (val.first()=='#') {
      do
        text.rd(forParse);
      while (forParse != '\n');
    }
    else {
      intVal.setImage(val);
      maxGrey=intVal.asLong();
      break;
    }
  }
  while (1);

  PICLONG bytesToRead = width*height*3;
  dataBuf = (unsigned char *)picAllocate(bytesToRead+1);
  if(!dataBuf)
    return NULLPTR;

  if (strcmp(magicValue, "P3")==0) {
    DEBUG_ALG("Reading P3\n");
    double midVal;
    for (PICLONG i=0; i<bytesToRead; i++) {
      text.rd(val, ' ');
      intVal.setImage(val);
      midVal=(double)intVal.asLong() / (maxGrey==256 ? maxGrey-1 : maxGrey)
        * 255; //Kludge for the screen grabber because of its poor code
      *(dataBuf+i)=(unsigned char)midVal;
    }
    raw=new PicRGBPicture(width, height, dataBuf);
  }else{
    PICSIZE bytesInBuffer=0, size;

    // This check is needed because we used PicText :-(
    if(from.IS_A(PicBufferStream)){
      PicBufferStream &bstr = (PicBufferStream &)from;
      char *current = bstr.readAhead(size);
      bytesInBuffer = bstr.bytesInBuffer();
      if(current){
        bytesToRead -= bstr.bytesInBuffer();  
        memcpy(dataBuf, current, bstr.bytesInBuffer());
      }
      bytesToRead-=bytesInBuffer;
      if(bytesToRead < 0) {
        bytesToRead=0;
      }
    }
    from.read(dataBuf+bytesInBuffer, bytesToRead);
    DEBUG_ALG("Width=" & asString(width) & "\n");
    DEBUG_ALG("Height=" & asString(height) & "\n");

    raw=new PicRGBPicture(width, height, dataBuf);
  }
  picFree(dataBuf);
  return raw;
}

PICBOOL PicPPM::writePicture(PicStream &out, PicRawPicture *raw)
{
  if(!raw || raw->isNull())
    return boolFalse;

  char *magicValue="P6\n";
  out.write(magicValue, 3);

  BaseString value(asString(raw->getWidth()));
  out.write(value.chars(), value.size());
  out.write(" ", 1);

  BaseString value2(asString(raw->getHeight()));
  out.write(value2.chars(), value2.size());
  out.write("\n", 1);

  char *maxGrey="255\n";
  out.write(maxGrey, 4);

  out.write(raw->getDataPtr(), raw->getWidth() * raw->getHeight() * 3);

  return boolTrue;
}

