// **********************************************************************
//  This file contains PicJpeg implementation
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <pfilestm.hxx>
#include <PicImageFormat.hxx>
#include <mmain.hxx>

//#ifdef _WIN32
extern "C" {
  //#endif
#include <jpeglib.h>
#include <jinclude.h>
#include <jerror.h>
#include <setjmp.h>
#include <stdio.h>
  //#ifdef _WIN32
}
//#endif

MAKE_PICOBJECTCLASS(PicJpeg, PicImageFormat)

PicJpeg::PicJpeg()
{
  name = "JPEG";
  id = PIC_JPEG_IMAGEFORMAT_ID;
  fileExtensions.add("jpg");
  fileExtensions.add("jpeg");
  isLossy = boolTrue;
}

// **********************************************************************
//  Functions needed by Jpeg-library

struct my_error_mgr {
  struct jpeg_error_mgr pub; /* "public" fields */

  jmp_buf setjmp_buffer; /* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;

void my_error_exit (j_common_ptr cinfo)
{
  my_error_ptr myerr = (my_error_ptr) cinfo->err;
  longjmp(myerr->setjmp_buffer, 1);
}

// **********************************************************************
//  Source manager

typedef struct {
  struct jpeg_source_mgr pub;	/* public fields */

  PicStream *stream;
  JOCTET    *buffer;
  boolean   start_of_file;
} my_source_mgr;

typedef my_source_mgr * my_src_ptr;

#define INPUT_BUF_SIZE  4096  /* choose an efficiently fread'able size */

void my_init_source (j_decompress_ptr cinfo)
{
  my_src_ptr src = (my_src_ptr) cinfo->src;

  src->start_of_file = TRUE;
}

boolean my_fill_input_buffer (j_decompress_ptr cinfo)
{
  my_src_ptr src = (my_src_ptr) cinfo->src;
  PICSIZE    nbytes;
  
  nbytes = src->stream->read(src->buffer, INPUT_BUF_SIZE);

  if (nbytes <= 0) {
    if (src->start_of_file)	/* Treat empty input file as fatal error */
      ERREXIT(cinfo, JERR_INPUT_EMPTY);
    WARNMS(cinfo, JWRN_JPEG_EOF);
    /* Insert a fake EOI marker */
    src->buffer[0] = (JOCTET) 0xFF;
    src->buffer[1] = (JOCTET) JPEG_EOI;
    nbytes = 2;
  }
  src->pub.next_input_byte = src->buffer;
  src->pub.bytes_in_buffer = nbytes;
  src->start_of_file = FALSE;

  return TRUE;
}

void my_skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{
  my_src_ptr src = (my_src_ptr) cinfo->src;

  if (num_bytes > 0) {
    while (num_bytes > (long) src->pub.bytes_in_buffer) {
      num_bytes -= (long) src->pub.bytes_in_buffer;
      my_fill_input_buffer(cinfo);
    }
    src->pub.next_input_byte += (size_t) num_bytes;
    src->pub.bytes_in_buffer -= (size_t) num_bytes;
  }
}

void my_term_source (j_decompress_ptr cinfo)
{
  /* no work necessary here */
}

void my_jpeg_src (j_decompress_ptr cinfo, PicStream *stream)
{
  my_src_ptr src;

  if (cinfo->src == NULL) {	/* first time for this JPEG object? */
    cinfo->src = (struct jpeg_source_mgr *)
      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,
          SIZEOF(my_source_mgr));
    src = (my_src_ptr) cinfo->src;
    src->buffer = (JOCTET *)
      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,
                                   INPUT_BUF_SIZE * SIZEOF(JOCTET));
  }
  src = (my_src_ptr) cinfo->src;
  src->pub.init_source = my_init_source;
  src->pub.fill_input_buffer = my_fill_input_buffer;
  src->pub.skip_input_data = my_skip_input_data;
  src->pub.resync_to_restart = jpeg_resync_to_restart; /* use default method */
  src->pub.term_source = my_term_source;
  src->stream = stream;
  src->pub.bytes_in_buffer = 0; /* forces fill_input_buffer on first read */
  src->pub.next_input_byte = NULL; /* until buffer loaded */
}

// **********************************************************************
//  Destination manager

typedef struct {
  struct jpeg_destination_mgr pub; /* public fields */

  PicStream *stream;
  JOCTET    *buffer;
} my_destination_mgr;

typedef my_destination_mgr * my_dest_ptr;

#define OUTPUT_BUF_SIZE  4096	/* choose an efficiently fwrite'able size */

void my_init_destination (j_compress_ptr cinfo)
{
  my_dest_ptr dest = (my_dest_ptr) cinfo->dest;

  /* Allocate the output buffer --- it will be released when done with image */
  dest->buffer = (JOCTET *)
      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_IMAGE,
                                  OUTPUT_BUF_SIZE * SIZEOF(JOCTET));

  dest->pub.next_output_byte = dest->buffer;
  dest->pub.free_in_buffer = OUTPUT_BUF_SIZE;
}

boolean my_empty_output_buffer (j_compress_ptr cinfo)
{
  my_dest_ptr dest = (my_dest_ptr) cinfo->dest;

  if (dest->stream->write(dest->buffer, OUTPUT_BUF_SIZE) != OUTPUT_BUF_SIZE)
    ERREXIT(cinfo, JERR_FILE_WRITE);

  dest->pub.next_output_byte = dest->buffer;
  dest->pub.free_in_buffer = OUTPUT_BUF_SIZE;

  return TRUE;
}

void my_term_destination (j_compress_ptr cinfo)
{
  my_dest_ptr dest = (my_dest_ptr) cinfo->dest;
  size_t datacount = OUTPUT_BUF_SIZE - dest->pub.free_in_buffer;

  if (datacount > 0) {
    if(dest->stream->write(dest->buffer, datacount) != datacount)
      ERREXIT(cinfo, JERR_FILE_WRITE);
  }
  if(dest->stream->IS_A(PicBufferStream)){
    PicBufferStream *bstr = (PicBufferStream *)(dest->stream);
    bstr->flushBuffer();
  }
}

void my_jpeg_dest (j_compress_ptr cinfo, PicStream* dest_stream)
{
  my_dest_ptr dest;

  if (cinfo->dest == NULL) {	/* first time for this JPEG object? */
    cinfo->dest = (struct jpeg_destination_mgr *)
      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,
                                  SIZEOF(my_destination_mgr));
  }
  dest = (my_dest_ptr) cinfo->dest;
  dest->pub.init_destination = my_init_destination;
  dest->pub.empty_output_buffer = my_empty_output_buffer;
  dest->pub.term_destination = my_term_destination;
  dest->stream = dest_stream;
}

// **********************************************************************

PicRawPicture *PicJpeg::readPicture(PicStream &from, PICULONG size)
{
  PicRawPicture *ans = NULLPTR;

  struct jpeg_decompress_struct cinfo;
  struct my_error_mgr           jerr;

  unsigned char *buffer[1], *temp, *temp2;
  int           row_stride;
  PICBOOL       grayScale = boolFalse;

  DEBUG_ALG("Reading Jpeg...\n");

  cinfo.err = jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit = my_error_exit;
  if (setjmp(jerr.setjmp_buffer)) {
    /* If we get here, the JPEG code has signaled an error.
     * We need to clean up the JPEG object, close the input file, and return.
     */
    jpeg_destroy_decompress(&cinfo);
    return NULLPTR;
  }
  jpeg_create_decompress(&cinfo);
  my_jpeg_src(&cinfo, &from);

  jpeg_read_header(&cinfo, TRUE);

  jpeg_start_decompress(&cinfo);
  row_stride = cinfo.output_width * cinfo.output_components;
  /* Make a one-row-high sample array that will go away when done with image */
//    buffer = (*cinfo.mem->alloc_sarray)
//             ((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

  if(cinfo.jpeg_color_space == JCS_GRAYSCALE)
    grayScale = boolTrue;
    
  ans = new PicRGBPicture();
  ans->setWidth(cinfo.output_width);
  ans->setHeight(cinfo.output_height);
  ans->setDataPtr(
    (unsigned char *)picAllocate(ans->getWidth() * ans->getHeight() * 3));

  /* Here we use the library's state variable cinfo.output_scanline as the
   * loop counter, so that we don't have to keep track ourselves.
   */
  while (cinfo.output_scanline < cinfo.output_height) {
    /* jpeg_read_scanlines expects an array of pointers to scanlines.
     * Here the array is only one element long, but you could ask for
     * more than one scanline at a time if that's more convenient.
     */
     buffer[0] = ans->getDataPtr() + cinfo.output_scanline*cinfo.output_width*3;
     jpeg_read_scanlines(&cinfo, buffer, 1);
     if(grayScale){
        temp = buffer[0] + cinfo.output_width - 1;
        temp2 = buffer[0] + cinfo.output_width*3 - 1;
        while(temp >= buffer[0]){
            *temp2 = *temp; temp2--;
            *temp2 = *temp; temp2--;
            *temp2 = *temp; temp2--;
            temp--;
        }
     }
  }
  jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);
  DEBUG_ALG("Read Jpeg ok.\n");

  return ans;
}

// **********************************************************************

PICBOOL PicJpeg::writePicture(PicStream &to, PicRawPicture *raw)
{
  struct jpeg_compress_struct cinfo;
  struct my_error_mgr         jerr;

  JSAMPROW row_pointer[1]; /* pointer to JSAMPLE row[s] */
  int      row_stride; /* physical row width in image buffer */

  if(!raw || raw->isNull())
    return boolFalse;

  cinfo.err = jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit = my_error_exit;
  if (setjmp(jerr.setjmp_buffer)) {
    /* If we get here, the JPEG code has signaled an error.
     * We need to clean up the JPEG object, close the output file, and return.
     */
    jpeg_destroy_compress(&cinfo);
    return NULLPTR;
  }
  jpeg_create_compress(&cinfo);
  my_jpeg_dest(&cinfo, &to);

  cinfo.image_width = raw->getWidth();
  cinfo.image_height = raw->getHeight();
  cinfo.input_components = 3;
  cinfo.in_color_space = JCS_RGB;
  jpeg_set_defaults(&cinfo);

//  jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);
  jpeg_start_compress(&cinfo, TRUE);

  /* Here we use the library's state variable cinfo.next_scanline as the
   * loop counter, so that we don't have to keep track ourselves.
   */
  row_stride = cinfo.image_width * 3; /* JSAMPLEs per row in image_buffer */
  while (cinfo.next_scanline < cinfo.image_height) {
    /* jpeg_write_scanlines expects an array of pointers to scanlines.
     * Here the array is only one element long, but you could pass
     * more than one scanline at a time if that's more convenient.
     */
    row_pointer[0] = raw->getDataPtr() + cinfo.next_scanline * row_stride;
    jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  jpeg_finish_compress(&cinfo);
  jpeg_destroy_compress(&cinfo);

  return boolTrue;
}

