// **********************************************************************
//  This file contains PicSearchAlgorithm implementation
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997-1999 PICSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <PicSearchAlgorithm.hxx>
#include <dlfcn.h>									// dlopen() & co
#include <iostream.h>

PicSearchAlgorithm::PicSearchAlgorithm(const char *libPath, int id) 
{
	alg_id=0;								// Until success is sure

	libhandle=dlopen(libPath, RTLD_NOW);    // Open algorithm library
	if(!libhandle)
	{
		cout << "Alg lib error: " << libPath << ":\n";
		cout << "Alg lib error: " << dlerror() << "\n";
		return;
	}

	// Assign functions
	
	compare=(PICUSHORT (*)(PicSignature &firstSig, PicSignature &secSig))
			dlsym(libhandle, "compare");
	if(!compare)
	{
		cout << "Alg lib error: " << dlerror() << "\n";
		dlclose(libhandle);
		libhandle=NULL;
		return;
	}

	computeSignature=(PicSignature *(*)(PicPicture &Pic))
						dlsym(libhandle, "computeSignature");
	if(!computeSignature)
	{
		cout << "Alg lib error: " << dlerror() << "\n";
		dlclose(libhandle);
		libhandle=NULL;
		return;
	}
	
	setParams=(PICBOOL (*)(PicIxCollect &params))
				dlsym(libhandle, "setParams");
	if(!setParams)
	{
		cout << "Alg lib error: " << dlerror() << "\n";
		dlclose(libhandle);
		libhandle=NULL;
		return;
	}
	algName=(char *(*)(void))dlsym(libhandle, "algName");
	if(!algName)
	{
		cout << "Alg lib error: " << dlerror() << "\n";
		dlclose(libhandle);
		libhandle=NULL;
		return;
	}
	algInfo=(char *(*)(void))dlsym(libhandle, "algInfo");
	if(!algInfo)
	{
		cout << "Alg lib error: " << dlerror() << "\n";
		dlclose(libhandle);
		libhandle=NULL;
		return;
	}
	alg_id=id;
}

PicSearchAlgorithm::~PicSearchAlgorithm()
{
	if(libhandle)
	{
		dlclose(libhandle);
	}
}

// **********************************************************************
