// **********************************************************************
//  This file contains PicSegmentation and PicSegment implementations
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <palloc.hxx>
#include <PicSegment.hxx>

MAKE_PICOBJECTCLASS(PicSegmentation, PicObject)

PicSegmentation::PicSegmentation() :
  algId(0)
{
}

PicSegmentation::~PicSegmentation()
{
  setNull();
}

PicSegmentation::PicSegmentation(const PicSegmentation &other) :
  algId(0)
{
  *this = (const PicObject &)other;
}

PicObject& PicSegmentation::operator = (const PicObject& obj)
{
  if(obj.IS_A(PicSegmentation)){
    const PicSegmentation &other = AS(PicSegmentation, obj);

    segments = other.segments;
    algId = other.algId;
  }
  return *this;
}

PICBOOL PicSegmentation::operator == (const PicObject& other) const
{
  return (this == &other);
}

PICBOOL PicSegmentation::operator <  (const PicObject&) const
{
  undefined("PicSegmentation::operator <");
  return boolFalse;
}

PICBOOL PicSegmentation::isNull() const
{
  return algId == 0;
}

void PicSegmentation::setNull()
{
  segments.setNull();
  algId = 0;
}

BaseString PicSegmentation::getImage() const
{
  return "Segmentation (alg=" & asString(algId) & ", segments: " & 
          asString(segments.size());
}

void PicSegmentation::setImage(const BaseString& image)
{
  undefined("PicSegmentation::setImage");
}

// **********************************************************************

MAKE_PICOBJECTCLASS(PicSegment, PicObject)

PicSegment::PicSegment() :
  dataSize(0), data(NULLPTR)
{
}

PicSegment::~PicSegment()
{
  setNull();
}

PicSegment::PicSegment(const PicSegment &other) :
  dataSize(0), data(NULLPTR)
{
  *this = (const PicObject &)other;
}

PicObject& PicSegment::operator = (const PicObject& obj)
{
  if(obj.IS_A(PicSegment)){
    const PicSegment &other = AS(PicSegment, obj);

    setNull();
    PICULONG size = other.dataSize;
    if(size > 0){
      data = (char *)picAllocate(size);
      if(data){
        memcpy(data, other.data, size);
        dataSize = other.dataSize;
      }
    }
  }
  return *this;
}

PICBOOL PicSegment::operator == (const PicObject& other) const
{
  return (this == &other);
}

PICBOOL PicSegment::operator <  (const PicObject&) const
{
  undefined("PicSegment::operator <");
  return boolFalse;
}

PICBOOL PicSegment::isNull() const
{
  return data == NULLPTR;
}

void PicSegment::setNull()
{
  if(data)
    picFree(data);
  dataSize = 0;
  data = NULLPTR;
}

BaseString PicSegment::getImage() const
{
  return "Segment of size " & asString(dataSize);
}

void PicSegment::setImage(const BaseString& image)
{
  undefined("PicSegment::setImage");
}

