// **********************************************************************
//  Useful macros for debugging etc.
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef MERRORS_HXX
#define MERRORS_HXX

#include <picerror.hxx>

//:macro ********************************************************************

#define DEBUG_GUI_ON PicMain::globData.picPars->picGUIDebug
#define DEBUG_GUI_OFF !PicMain::globData.picPars->picGUIDebug
#define DEBUG_CORE_ON PicMain::globData.picPars->picCoreDebug
#define DEBUG_CORE_OFF !PicMain::globData.picPars->picCoreDebug
#define DEBUG_ALG_ON PicMain::globData.picPars->picAlgDebug
#define DEBUG_ALG_OFF !PicMain::globData.picPars->picAlgDebug
#define DEBUG_DB_ON PicMain::globData.picPars->picDbDebug
#define DEBUG_DB_OFF !PicMain::globData.picPars->picDbDebug
// New in v3
#define DEBUG_CACHE_ON PicMain::globData.picPars->picCacheDebug
#define DEBUG_CACHE_OFF !PicMain::globData.picPars->picCacheDebug

/*
Use these macros to find out whether your subsystem has a debug
option on. 
This may be used for example to control whether the user will see 
primitive level errors produced by your subsystem, i.e. whether you 
use ERAISE (debug is off) or EHOLD after detecting a low level error. 
E.g.
  myOtherFunc(str);   // a primitive function 
  if(PicError::isDetected()){
    if(DEBUG_xxx_OFF){
        // overwrite the actual cause by a more general description
        ERAISE(MyClass, generalFailure);  
    }else{
        EHOLD(MyClass, generalFailure);
    }
    return;
  }
*/

//:macro ********************************************************************

#define DEBUG_GUI(MSG) if(DEBUG_GUI_ON){ PicMain::debugMesgGUI(MSG); }
#define DEBUG_CORE(MSG) if(DEBUG_CORE_ON){ PicMain::debugMesgCore(MSG); }
#define DEBUG_ALG(MSG) if(DEBUG_ALG_ON){ PicMain::debugMesgAlg(MSG); }
#define DEBUG_DB(MSG) if(DEBUG_DB_ON){ PicMain::debugMesgDb(MSG); }
// New in v3
#define DEBUG_CACHE(MSG) if(DEBUG_CACHE_ON) { PicMain::debugMesgCache(MSG); }

/*
Use these macros to print debug messages for your subsystem.

arg: MSG must be of type 'BaseString' or '(const) char *'.

E.g. DEBUG_GUI("Gui initialized ok.\n");
     DEBUG_CORE("Query thread started at " & BaseTime(now).getImage() & "\n");
*/

//:macro ********************************************************************

#define INFO_PRINT(MSG) PicMain::getGUI().infoPrint(MSG);
#define INFO_PRINTLN(MSG) PicMain::getGUI().infoPrint(MSG & BaseString("\n"));
/*
Use these macros to print a message in the info window.
INFO_PRINTLN appends a newline to your string, INFO_PRINT does not.

arg: MSG must be of type 'BaseString' or '(const) char *'.

E.g. INFO_PRINTLN("Query started.");
     INFO_PRINT("Query started with algorithm" & algName & ".\n");
*/

//:macro ********************************************************************

#define PRINT_ERRORS() PicMain::printErrors();
#define FLUSH_ERRORS() if(1){ PicMain::printErrors(); PicError::clearCurrent(); }
/*
Use these macros to print current errors onto the info window.
FLUSH_ERRORS clears the errors after printing them out.
*/

#endif
