// **********************************************************************
//  Defintions of threads for PicCore
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef MTHREADS_HXX
#define MTHREADS_HXX

#include <pthread.hxx>
#include <parray.hxx>
#include <pfile.hxx>
#include <btime.hxx>
#include <mquery.hxx>

/*:class ********************************************************************

PicEnhancedThread adds a clean way to suspend/abort a thread when 
it's running. This is achieved through a public command variable
that can be set by anyone. The actual thread implementation can check
occasionally the status of the variable with checkCommand function 
and exit/suspend action if so requested.

****************************************************************************/

class PicEnhancedThread : public PicThread
{
 public:

  PICVIRTCLASS(PicEnhancedThread)
  PicEnhancedThread(PicPriority priority = normal);

//:var
  enum PicThreadCmd
  {
     go,
     suspend,
     isSuspended,
     abort
  }command;
/*
Set the value of this variable to control thread's activity:
  go          -- free to run 
  suspend     -- should suspend all action
  isSuspended -- thread sets command from suspend ==> isSuspended 
  abort       -- should halt
*/

//:var
  BaseString name;
/*
Name for "the function the thread is for".
*/

};

/*:class ********************************************************************

Mass insert thread implements the batch insertion of images into 
database. It reads the pictures from files specified by a special 
list file or a simple file pattern (e.g. "*.jpg").
A list file should contain filenames or masks (one per line).
Each line can optionally have strings for name, par1 and par2 
attributes for the image(s). E.g.
    myface.jpg # Me # Apr 1997 # people
    /images/chr96/c*.jpg # Christmas # Dec 1997

****************************************************************************/

class PicMassInsertThread : public PicEnhancedThread
{
 public:

  PICVIRTCLASS(PicMassInsertThread)
  PICERRORCODES {  ok,
    noFileName,
    fileNotExist
  };

  PicMassInsertThread(PicPriority priority = normal);

//:var
  PicFile listFile;
/*
Main thread sets listFile before starting the thread. If it has 
suffix "lst" (case-insensitive) it is considered a list file
and will be opened to read the actual filenames of the images.
Otherwise it is taken to mean the actual file(s) to be inserted
into database.
*/

 protected:

  virtual PICTHREADRESULT threadFunction(PICTHREADARG arg);

  virtual PICBOOL doInits();
  virtual PICBOOL printStats();
  virtual PICBOOL extractParams(BaseString line, BaseString &fileName, 
                                PicPicture &pic);
  virtual PICBOOL insertPic(PicPicture &pic);

  PICULONG   insertedPics;
  PICULONG   failedPics;
  PICULONG   notFound;
  PicStream  *inputStream;
  BaseString tempBuf;  

};

#endif

