// **********************************************************************
//  This file contains the database related routines of PicMain
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <mmain.hxx>
#include <PicSignature.hxx>

#include <PICClient.h>

// **********************************************************************

PICBOOL PicMain::browseQueryResults(PICUSHORT numOfFirst)
{
  DEBUG_CORE("Browsing query results, from " & 
             asString(numOfFirst) & " onwards.\n");

  // NETP: Using PICClient send request for thumbnails
  //       Note: see PicQueryThread::sendResults 
  //       (defined in mthreads.cxx) how to send the thumbnails
  //       to GUI. This action should take place somewhere
  //       in PICClient, where it receives the thumbnails 
  //       from the server
  
  PICBOOL ret = boolTrue;

  PicArray genOpts;
  ret = getParams(genOpts);
  if(ret){
    ret = PICClient::GetMoreThumbNails(numOfFirst, genOpts);
  }
//  PICBOOL ret = queryThread.sendResults(numOfFirst, boolFalse, db);
  FLUSH_ERRORS();
  return ret;
}

// **********************************************************************

PICBOOL PicMain::suspendQuery()
{
  return boolFalse;
}

// **********************************************************************

PICBOOL PicMain::resumeQuery()
{
  return boolFalse;
}

// **********************************************************************

PICBOOL PicMain::insertImageToDb(PicPicture &image, PICULONG listIndex,
  PICBOOL updating)
{
  PicErrorContext ecxt;

  PICBOOL doingOk = boolTrue; 

  // Because of our stupid database:
  if(batchThread.state() == PicThread::running){
    INFO_PRINT("Sorry, batch insertion blocking others.\n");
    return boolFalse;
  }

  PicPicture *pic = getExamplePic(listIndex);

  if(pic){
    if(pic->raw)
      delete pic->raw;
    pic->raw = image.raw;  // Lend the pointer for a while
    if(pic->origData.isNull()){
      BaseString strBuffer;
      PicStringStream stream(strBuffer);

      pic->format = globData.getImageFormat(PIC_JPEG_IMAGEFORMAT_ID);
      pic->format->writePicture(stream, pic->raw);
      pic->origData = strBuffer;  // PicBin::operator= for BaseString
    }
    doingOk = PICClient::InsertImageToDb(*pic);
    
    pic->raw = new PicRawPicture();
    pic->raw->setWidth(image.raw->getWidth());
    pic->raw->setHeight(image.raw->getHeight());

    // pic->id should ideally get the new image id ...

    image.id = pic->id;
  }

  FLUSH_ERRORS();
  return doingOk;
}

// **********************************************************************

/*
PICBOOL PicMain::readImageFromDb(PICULONG imageId,
                                 PICULONG listIndex,
                                 PicPicture &resultImage)
{
  PicErrorContext ecxt;

  if(imageId == 0){
    INFO_PRINT("Unknown image.\n");
    return boolFalse;
  }
  resultImage.setNull();
  PicPicture *pic = getExamplePic(listIndex);

  PICBOOL doingOk = boolFalse;
  if(pic){
    PicPicture backUp(*pic);
    doingOk = boolTrue;

    // NETP: here we should call PICClient::readxxx(imageId, resultImage)
    //       to read the image and synchronously(!) wait for 
    //       the data to arrive. If the reading is not synchronous
    //       we should somehow modify the GUI-code to receive an
    //       image at arbitrary time. This would look pretty weird
    //       as the image would pop into the query image area and
    //       replace(!?) an existing query image.

    doingOk = PICClient::ReadImageFromDb(imageId, resultImage);

    // (should set into resultImage all the values that 
    // are copied below into pic
    DEBUG_CORE("Image read from database with image id " & 
               asString(imageId) & ".\n");
    if(doingOk){
      DEBUG_CORE("Image id is " & asString(resultImage.id) & ".\n");
      pic->id = resultImage.id;
      pic->name = resultImage.name;
      pic->par1 = resultImage.par1;
      pic->par2 = resultImage.par2;
      // image data should be read emulating as closely as possible
      // the functionality of PicMainDbUser::readImageData
      // (defined in main2db.cxx)
      if(doingOk && pic->raw && resultImage.raw){
        pic->raw->setWidth(resultImage.raw->getWidth());
        pic->raw->setHeight(resultImage.raw->getHeight());
      }
      pic->filename = resultImage.filename;
      pic->format = resultImage.format;
      pic->identicalToFilename = resultImage.identicalToFilename;
      pic->origData = (PicObject &)(resultImage.origData);
    }
    // Signatures we do not sadly get any more, so we can't
    // browse them in "Image attributes" dialog window

    if(!doingOk){
      resultImage.setNull();
      *pic = (const PicObject &)backUp;
      EHOLD(PicMain, imageReadFailed);
    }
  }
  FLUSH_ERRORS();
  return doingOk;
}
*/

// **********************************************************************

PICBOOL PicMain::deleteImageFromDb(PICULONG imageId,
                                   PICUSHORT buttonNbr)
{
  PicErrorContext ecxt;

  if(imageId == 0){
    INFO_PRINT("Delete: Image not in database.\n");
    return boolFalse;
  }
  // Because of our stupid database:
  if(batchThread.state() == PicThread::running){
    INFO_PRINT("Sorry, batch insertion blocking others.\n");
    return boolFalse;
  }
  INFO_PRINT("Removing image (id=" & asString(imageId) & ") from database.\n");

  // NETP: call PICClient::DeleteImageFromDb(imageId) to remove the image
  //       We've got the same trouble here as in query, since
  //       we need to no the server+database id also, imageId
  //       by itself is not any more unique key.

  if(PICClient::DeleteImageFromDb(imageId))
  {
    PicIterator iter;
    ITERATE(examplePics, iter)
    {
      PicPicture &curr = AS(PicPicture, examplePics.at(iter));
      if(curr.id == imageId){
        curr.id = 0;
        curr.signatures.listClean();
        curr.segmentations.deleteAll();
      }
    }
  }
  FLUSH_ERRORS();
  
  return boolTrue;
}

// **********************************************************************

PICBOOL PicMain::batchInsert(BaseString filename, PICBOOL abortBatch)
{
  PicErrorContext errcxt;
  PICBOOL ret = boolFalse;

  if((batchThread.state() == PicThread::running) && !abortBatch){
    return boolFalse;

  INFO_PRINT("Starting batch insertion...\n");

  }  
  abortThread(batchThread);
  if(batchThread.state() == PicThread::running){
    return boolFalse;
  }else{
    batchThread.wait();
    batchThread.command = PicEnhancedThread::go;
    batchThread.listFile.setFullName(filename.copy());

    ret = batchThread.start(NULLPTR);  // do not use the arg!
  }
  FLUSH_ERRORS();
  return ret;
}

