// --------------------------------------------------------------------------
//  CHARACTER
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

#ifndef PCHAR_HXX
#define PCHAR_HXX

#include <picbase.hxx>
#include <picerror.erh>

/*:class *** CHARACTER ******************************************************

PicChar is a class that collects all the character-related methods together.
Instances of PicChar are never created.

****************************************************************************/

class PicChar
{
    public:

    PICCLASS()

// --------------------------------------------------------------------------
//:enum

    enum CompareResult { isLess = -1, isEqual = 0, isGreater = 1 };

/*
Possible results of a general comparison function.
*/

// --------------------------------------------------------------------------
//:fun

    static char uppercase      (char);
    static char lowercase      (char);

    static PICBOOL isUppercase    (char);
    static PICBOOL isLowercase    (char);
    static PICBOOL isAlphabetic   (char);
    static PICBOOL isAlphanumeric (char);
    static PICBOOL isDigit        (char);
    static PICBOOL isSpace        (char);

/*
PicChar classification recognizes four base classes:

  * uppercase characters
  * lowercase characters
  * whitespace characters
  * digits (0-9)

In addition two combination classes are recognized:

  * alphabetic (lowercase and uppercase characters)
  * alphanumeric (alphabetic and digits)
*/

// --------------------------------------------------------------------------
//:fun

    static CompareResult caseCompare (char, char);
    static CompareResult compare     (char, char);

/*
The method compare is case insensitive; caseCompare is case sensitive. 
The characters' ASCII values are used in comparision.

:ret isEqual:   Argument 1 == Argument 2.
:ret isLess:    Argument 1 <  Argument 2.
:ret isGreater: Argument 1 >  Argument 2.

*/

};

typedef PicChar::CompareResult PicCompareResult;


#endif
