// --------------------------------------------------------------------------
//  PicEnvironment
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

#ifndef PENVIRON_HXX
#define PENVIRON_HXX

#include <bstring.hxx>

/*:class ********************************************************************

PicEnvironment provides read-only access to application environment through
static methods. It contains a set of environment variables. 
Environment variable has name and textual value, expressed as name=value pair.

****************************************************************************/

class PicEnvironment
{
    public:
// --------------------------------------------------------------------------
//:fun  METHODS

    static BaseString programName ();

//:fun ----------------------------------------------------------------------

    static BaseString arguments ();

//:fun ----------------------------------------------------------------------

    static BaseString value (const char* varName);
    static BaseString value (const BaseString& varName);

/*
These static methods get the value of the requested environment variable.

:arg const char*: Name of the environment variable.
:arg const BaseString&: Name of the environment variable.
:ret BaseString: Value of the requested variable, null string if it was not
                 found.
*/
//:fun ----------------------------------------------------------------------

    static void replaceEnvironmentValues (BaseString&   replaced,
                                          const char* separator = NULLPTR);
/*
This static method replaces all environment variables with their values
like in script files (%VARIABLE_NAME%). If Environment variable APPLDRV=D:,
String "%APPLDRV%\FILE.TXT" is changed to "D:\FILE.TXT".
'%'-is default separator, but any strings can be used. Replaced string
can contain many environment variables.
If some environment variable is not defined, it is replaced with empty string. 
If separator is two times successive, one is removed
  (" %%-char" -> " %-char" )
      
:arg const BaseString&: Variable to replaced.
:arg const char:        Separator, if NULLPTR, '%' is used.
*/
    
};

// --------------------------------------------------------------------------
//  INLINE DEFINITIONS
// --------------------------------------------------------------------------

inline BaseString PicEnvironment::value (const BaseString& varName)
       { return value(varName.chars()); }

#endif
