// --------------------------------------------------------------------------
//  ERROR CONTEXT
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

#ifndef PERRCTXT_HXX
#define PERRCTXT_HXX

#include <picbase.hxx>
class PicError;
class BaseString;
class PicObject;
class PicText;
class PicFmtText;

/*:class *** ERROR CONTEXT **************************************************

PicErrorContext objects contain error information. They are created as local
automatic objects and they are linked together to form a context stack. The
context stack divides the program in sections which are protected from each
others errors.

Creating an automatic PicErrorContext object enters a new error context.
The destructor automatically exits the context. For example:

    {
        PicErrorContext ecxt; // enter the context
        ...
        // exit the context
    }

****************************************************************************/

class PicErrorContext
{
    public:
// --------------------------------------------------------------------------

    PicErrorContext ();

//:fun ----------------------------------------------------------------------

   ~PicErrorContext ();

/*
Destructor exits current context. 
*/

    PICCLASS()

    private:
// ==========================================================================
//  INSTANCE VARIABLES

    PicErrorContext* previous;    // Points to the previous context.
    PicError*        error;       // Points to the error information.

    friend class PicErrorManager;
    friend class PicL1ErrorManager;
};


/*:class ********************************************************************

PicTestContext is a special error context that you use when you want
to test whether some operation succeeds without writing to the error
log or returning in error status. If you create a test context instead
of error context, the error log is turned off 
and the error status is cleared when the context is destroyed.

:abs Error context for testing whether something succeeds.

****************************************************************************/

class PicTestContext : public PicErrorContext
{
    public:

    PICCLASS()

// --------------------------------------------------------------------------
//:fun  CONSTRUCTORS, DESTRUCTOR & ASSIGNMENT

    PicTestContext ();
   ~PicTestContext ();

/*
The constructor turns off the error log. The destructor turns it on again
and clears any errors raised or transferred in the context.
*/

};

#endif
