// --------------------------------------------------------------------------
//  MEMORY ALLOCATION implementation
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

extern "C" {
#include <malloc.h>
#include <string.h>
}

#include <palloc.hxx>

#include <ptext.hxx>
#include <picerror.hxx>
#include <psemaph.hxx>

static PicSemaphore picAllocSemaph;


PICUINT picCalculateSize (PICUINT bytes)
{
    PICUINT alloc;

    if (bytes < 4096)
    {
        alloc = 16;

        while (alloc < bytes)
            alloc <<= 1;
        return alloc;
    }
    else
    {
        // Round the size to next even 2048 bytes
        // but check for the overflow case:

        alloc = ((bytes - 1) | 2047) + 1;
        return (alloc) ? alloc : maxNewSize;
    }
}

void* picAllocate (PICUINT bytes, PICBOOL clear)
{
    void* buffer;

    PicCritSec crsec(picAllocSemaph);

    buffer = (void *)(new char[bytes]);
    if (buffer)
    {
        if (clear)
            memset(buffer, 0, (size_t)bytes);
        return buffer;
    }
    else
    {
        PicError::raiseMemoryError("*** picAllocate failed ***");
        return NULLPTR;
    }
}

void* picResize (
    void*     oldBuffer,
    PICUINT   oldBytes,
    PICUINT   newBytes,
    PICBOOL   clear)
{
    void* newBuffer;

    picAllocSemaph.set();
    newBuffer = (void *)(new char[newBytes]);
    picAllocSemaph.clear();

    if (newBuffer)
    {
        if (oldBuffer)
        {
            memcpy(newBuffer, oldBuffer,
                (newBytes < oldBytes) ? newBytes : oldBytes);
            picAllocSemaph.set();
            delete oldBuffer;
            picAllocSemaph.clear();
        }
        if (clear && (newBytes > oldBytes))
            memset((char*)newBuffer + oldBytes, 0, newBytes - oldBytes);
        return newBuffer;
    }
    else
    {
        PicError::raiseMemoryError("*** picResize failed ***");
        return NULLPTR;
    }
}

void picFree (void* buffer)
{
  if (buffer)
  {
    PicCritSec crsec(picAllocSemaph);
    
	delete buffer;
  }
}

