// --------------------------------------------------------------------------
//  PicEnvironment implementation
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

#include <stdlib.h>

extern "C" {
#include <string.h>
}

#include <penviron.hxx>

BaseString PicEnvironment::value (const char* varName)
{
    BaseString env = varName;
    env.makeUppercase();

    BaseString val(getenv(env.chars()));
    
	return val;
}

void PicEnvironment::replaceEnvironmentValues (
    BaseString& replaced,
    const char* separator)
{
    PICPOSITION startPos,endPos;
    if (replaced.isEmpty())
        return;
    BaseString search("%");
    if (separator)
        search= separator; 
    PICSIZE searchSize = search.size();
    startPos = replaced.positionOf(search,1);
    while (startPos != outOfRange)
    {
        endPos = replaced. positionOf(search,startPos+searchSize);
        if (endPos == outOfRange)
            break;
        if (endPos != startPos + searchSize)
        {
            BaseString val = PicEnvironment::value(
                    replaced.between(startPos+searchSize,endPos-1));
            replaced. replace(replaced.between(startPos,endPos+searchSize-1)
                            ,val);
            startPos = replaced.positionOf(search,startPos);
        }
        else
        {
            replaced.deleteBetween(startPos,startPos+searchSize-1);
            startPos = replaced.positionOf(search,endPos);
        }
    }
}


// --------------------------------------------------------------------------
//      ARGUMENTS & PROGRAM NAME
//
//      Command line format is :
//          program name\0[ arguments]\0
//
//      DosGetEnv returns selector to segment where the environment is stored
//      and offset at the beginning of the command line.

BaseString PicEnvironment::arguments ()
{
    return BaseString();
}

BaseString PicEnvironment::programName ()
{
    return BaseString();
}
