// **********************************************************************
//  This is an empty "Skeleton" library example to help you build
//  your own algorithm libraries for PICSearch.
//
//  Just replace the code of the functions with your own code
//  and tweak the PicSkeleton.hxx file. You can naturally add
//  any internal functions you need as global in this file.
// **********************************************************************
// Copyright (C) 1999 PICSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <PicSkeleton.hxx>

// **********************************************************************

/*
 * NOTE: PICSearch algorithm libraries do not too nicely follow 
 * the object paradigm. Each and every library source file specifies 
 * the same nonvirtual functions of the class "PicLibAlgorithm",
 * as specified in "library/inc/PicLibAlgorith.hxx".
 *
 * This works, because source codes of two different libraries
 * are never linked together to form a single object.
 *
 */

/*
 * Do a setup, install the information given to the caller about
 * this algorithm.
 *
 * You MUST have these two variables set.
 *
 */
PicLibAlgorithm::PicLibAlgorithm()
{
  name="Skeleton";
  info="This is a do-nothing bare-bones algorithm";
}

/*
 * This function is used by the caller to change
 * algorithm parameters. You can store the parameters
 * as global variables, for example.
 * 
 * You get the parameters as a "PicIxCollect" array
 * (as defined in "source/common/inc/parray.hxx").
 * It's the stuff provided by the Tk parameter window,
 * if you have one.
 *
 * You MUST have this function, but it can be a dummy one.
 *
 * NOTE: The parameters should NOT affect computeSignature(),
 * they should only affect the result of compare()
 *
 */
PICBOOL PicLibAlgorithm::setParams(PicIxCollect &params)
{
  return boolTrue;
}

/*
 * This constructor should change the given binary 
 * representation of your signature to some more easily
 * handled internal variable representation.
 *
 */
PicSkeletonSignature::PicSkeletonSignature(PicSignature *from)
{
	sigFailure=0;                       // No failure as default
	this->signature=NULL;				// We don't have any binary in
	this->signature_size=0;				// the signature created by this
	
	PICULONG size=from->signature_size; // Size of the binary we got
	char *data=from->signature;         // Pointer to the binary data

	// Check that the signature data is valid
	// if not, set sigFailure=1 and return

	// Convert the signature 
}

/*
 * This function compares two signatures and outputs
 * the match percentage.
 *
 * You MUST have this function.
 *
 */
PICUSHORT PicLibAlgorithm::compare(PicSignature &firstSig,
                                       PicSignature &secSig)
{
	int score=0;

	PicSkeletonSignature q(&firstSig);	// Create internal representations
	PicSkeletonSignature t(&secSig);    // from the binaries
	if(q.sigFailure || t.sigFailure)
		return(0);

	// Calculate the match (score) using your internal representation

  	return (PICUSHORT)(score*1000.0);
}

/*
 * This function computes a signature for a picture received
 * as a reference to "PicPicture" class.
 *
 * You MUST have this function.
 *
 */
PicSignature *PicLibAlgorithm::computeSignature(PicPicture &pic)
{
	int value1=0, value2=0;

	if(!pic.raw || pic.raw->isNull()){
		return(NULL);
	}

	// Calculate the signature from &pic to some variables
	
	// Convert the variables to bin:

	PicSkeletonSignature *tmp=new PicSkeletonSignature(value1, value2);
	if(tmp->sigFailure)
		return(NULL);
	
	PicSignature *signature=tmp;

	// Return the converted signature
  
	return(signature);
}

/*
 * A function like this is used to create a new binary signature 
 * from the argument variables.
 *
 */
PicSkeletonSignature::PicSkeletonSignature(int dummy, int empty)
{
    int sigFailure=0;                   // No failure as default
	int size=123;						// Size of your binary sig
	char *binary=new char[size];		// Allocate space for your signature

	// Convert the arguments received to the binary array.

	this->signature=binary;				// Assign the signature
	this->signature_size=size;          // And its size
}

/*
 * A standard constructor.
 *
 */
PicSkeletonSignature::PicSkeletonSignature()
{
}

/*
 * A standard destructor.
 *
 */
PicSkeletonSignature::~PicSkeletonSignature()
{
}


