// **********************************************************************
//  Search algorithm class definitions
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef PICIDALG_HXX
#define PICIDALG_HXX

#include <PicLibAlgorithm.hxx>

/**
  * Id-search algorithm implements a simple searching by image's id number.
  **/

PICULONG imageId;
  /**
    * Flag defining whether to use example image's ID when it is non-zero.
    **/
PICBOOL  useExImageId;

/**
  * PicIdSignature represents the signature created by PicIdSearchAlgorithm.
  * It simply contains the imageId.
  **/

class PicIdSignature : public PicSignature
{
 public:

  /**
    * The default constructor.
    **/
  PicIdSignature();
  /**
    * Constructor which sets the signature data.
    *
    * @param imageId Image ID for the signature.
    **/
  PicIdSignature(PICULONG imageId);
  PicIdSignature(PicSignature *from);
  /**
    * The destructor.
    **/
  virtual ~PicIdSignature();
  
  int sigFailure;

 private:
  /**
    * Image ID.
    **/
  PICULONG imageId;

  friend class PicLibAlgorithm;
};

#endif


