// **********************************************************************
//  PicWavelet library header
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef PICWAVELET_HXX
#define PICWAVELET_HXX

#include <PicLibAlgorithm.hxx>

/**
  * PicWaveletAlgorithm implements a search algorithm based on wavelet
  * coefficients. It is heavily based on the algorithm described in:
  *
  *   Jacobs et al. (1995). Fast Multiresolution Image Querying. 
  *   SIGGRAPH'95 
  **/

//**************************************************************************

#define NUM_OF_WAVELET_COEFS  60

struct coords
{
  PICUSHORT x;
  PICUSHORT y;
  double val;
};

struct xy
{
  PICUSHORT x;
  PICUSHORT y;
};

/**
  * PicWaveletSignature is a signature class for PicWaveletAlgorithm.
  * It contains (for each color channel (RGB)):
  * \begin{itemize}
  *   \item Average value.
  *   \item Number of positive and negative coefficients 
  *         (total number of coefficients is fixed).
  *   \item The coordinates of the coefficients.
  * \end{itemize}
  **/

class PicWaveletSignature : public PicSignature
{
	public:
		PicWaveletSignature();
		PicWaveletSignature(PicSignature *from);
		PicWaveletSignature(double newAvgVals[3],
			PICUSHORT newPosCoefs[3], PICUSHORT newNegCoefs[3],
			struct xy  newBigCoefs[3][NUM_OF_WAVELET_COEFS]);
		~PicWaveletSignature();
	
	int sigFailure;
	
	private:
		double      avgVals[3]; 
		PICUSHORT   posCoefs[3], negCoefs[3];
		struct xy   bigCoefs[3][NUM_OF_WAVELET_COEFS];

	friend class PicLibAlgorithm;
};

// **********************************************************************

#endif

