// **********************************************************************
//  Implementations of search algorithms
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <picerror.hxx>
#include <pstring.hxx>
#include <pinteger.hxx>
#include <PicIDAlg.hxx>
#include <mmain.hxx>

#include <iostream.h>

PicLibAlgorithm::PicLibAlgorithm()
{
  imageId=0;
  useExImageId=boolFalse;
  
  name = "IDSearch";
  info = "IDSearch algorith\n";
}

PICBOOL PicLibAlgorithm::setParams(PicIxCollect &params)
{
  if(params.contains(1)){
    PicObject &curr = params.at(1);
    if(TYPE_GUARD(PicString::classId(), curr)){
      PicString &parVal = AS(PicString, curr);
      if(parVal == BaseString("1"))
        useExImageId = boolTrue;
      else
        useExImageId = boolFalse;
    }
  }
  if(params.contains(2)){
    PicObject &curr = params.at(2);
    if(TYPE_GUARD(PicString::classId(), curr)){
      PicString &parVal = AS(PicString, curr);
      PicInteger newId(parVal);
      if(PicError::isDetected() && 
         PicError::current().id() == PicInteger::classId())
      {
        // ERAISE(PicSearchAlgorithm, invalidParameters);
      }else{
        imageId = newId.asLong();
      }
    }
  }
  return boolTrue;
}
  

PICUSHORT PicLibAlgorithm::compare(PicSignature &firstSig, 
                                        PicSignature &secSig)
{
  PICUSHORT retVal = 0;

    PICULONG refId = imageId;
	
    PicIdSignature baseSig(&firstSig);
    PicIdSignature actSig(&secSig);
    if(baseSig.sigFailure || actSig.sigFailure)
        return(0);
   
	cout << "IDAlg: Sig1 is " << baseSig.imageId << "\n";
	cout << "IDAlg: Sig2 is " << actSig.imageId << "\n";
	
	if(useExImageId)
      refId = baseSig.imageId;

    if(refId > actSig.imageId){
      retVal = 0;
    }else{
      PICULONG diff = actSig.imageId - refId;
      if(diff > 1000)
        diff = 1000;
      retVal = 1000 - diff;
    }

  return retVal;
}

PicSignature *PicLibAlgorithm::computeSignature(PicPicture &pic)
{
  PicIdSignature *tmp = new PicIdSignature(pic.id);
  if(tmp->sigFailure)
  	return(NULL);
 
  return tmp;
}

/*
BaseString PicLibAlgorithm::getImage() const
{
  return name & " [" & asString(id) & 
                "] (params: id=" & asString(imageId) & ")";
}
*/

// **********************************************************************

PicIdSignature::PicIdSignature()
  : imageId(0)
{
	sigFailure=0;
}

PicIdSignature::PicIdSignature(PICULONG imId)
{
	sigFailure=0;
	imageId=imId;

	char *data = new char[sizeof(PICULONG)];

	if(data)
		memcpy(data, (char *)&imageId, sizeof(PICULONG));
	else
	{
		sigFailure=1;
		return;
	}

	this->signature=data;
	this->signature_size=sizeof(PICULONG);
}

PicIdSignature::PicIdSignature(PicSignature *from)
{
    sigFailure=0;
	this->signature=NULL;
	this->signature_size=0;
	
	char *data=from->signature;
	int sigSize=from->signature_size;

	if(sigSize!=sizeof(PICULONG) || !data)
	{
		sigFailure=1;
		imageId=0;
		cout << "IDAlg: Sigsize mismatch (Size: "<< sigSize <<")\n";
		return;
	}

	imageId = (PICULONG)*data;
	return;
}

PicIdSignature::~PicIdSignature() 
{ 
}

/*
BaseString PicIdSignature::getImage() const
{
  return asString(imageId);
}
*/
