#include<cfgread.h>
#include<algo.h>

string itos(int i) {
  string s;

  do {
    s+=(i%10)+'0';
    i/=10;
  } while(i);

  reverse(s.begin(),s.end());
  return s;  
}

int stoi(const string& s) {
  if(s=="") //just make sure these are interpreted correctly.
    return 0;
  return atoi(s.c_str());
}

cfgIteratorBase::cfgIteratorBase(stringMapMap::iterator i) : 
  rep(i) { }
cfgIteratorBase& cfgIteratorBase::operator++(void) { /* prefix */
  ++rep;
  return *this;
}
cfgIteratorBase cfgIteratorBase::operator++(int) {  /* postfix */
  cfgIteratorBase tmp=*this;
  ++*this;
  return tmp;
}
bool cfgIteratorBase::operator==(const cfgIteratorBase& rhs) const { 
  return rep==rhs.rep; 
}
bool cfgIteratorBase::operator!=(const cfgIteratorBase& rhs) const { 
  return rep!=rhs.rep; 
}

cfgIterator::cfgIterator(stringMapMap::iterator i) : 
  cfgIteratorBase(i) { }
const string& cfgIterator::operator*(void) const { 
  return (*rep).first; 
} 
const string& cfgIterator::name(void) const { 
  return (*rep).first; 
}

cfgAlgIterator::cfgAlgIterator(stringMapMap::iterator i) : 
  cfgIteratorBase(i) { }
algorithmID_t cfgAlgIterator::operator*(void) const { 
  return (algorithmID_t)stoi((*rep).first); 
} 
algorithmID_t cfgAlgIterator::ID(void) const { 
  return (algorithmID_t)stoi((*rep).first);
}


configFile::configFile(string fName) : fileName(fName) {
  load(); //load in the data.
}

configFile::~configFile(void) { }

void configFile::save(void) {
  ofstream cfgFile(fileName.c_str());

  for(stringMapMap::iterator i=theMap.begin();i!=theMap.end();++i) {
    cfgFile << "[" << (*i).first << "]" << endl;
    for(stringMap::iterator j=(*i).second.begin();j!=(*i).second.end();++j) {
      cfgFile << (*j).first << " = " << (*j).second << endl;
    }
    cfgFile << endl;
  }
}

void configFile::load(void) {
  ifstream cfgFile(fileName.c_str());
  string section;
  char buf[MAXCONFIGLINE];

  if(!cfgFile) {
    cerr << "failure to open " << fileName << " for configFile read" << endl;
    return;
  }

  /* here's the bit that does the reading. Sorry if it's ugly as hell. */
  while(skipSpace(cfgFile)) {
    cfgFile.getline(buf,MAXCONFIGLINE);
    int len=cfgFile.gcount()-1; //buf[cfgFile.gcount()-1]==0. skip the 0.

    if(buf[0]=='#') //I'll take this line as a comment.
      continue;

    if(buf[0]=='[') {  //Section head.
      char *pos=::find(&buf[1],&buf[len],']');  //find ']', ignore the rest.
      if(pos==&buf[len]) { //end of string, no ']'.
	cerr << "format error in configFile " << fileName << endl;
	continue;
      }
      section=string(&buf[1],pos-&buf[1]); //exclude '['.

      if(section[0]=='-') { //minus means 'remove section'.
	//section.erase(0,1); //remove first character (the '-'). //old lib..
	section.remove(0,1);
	theMap.erase(section); //erase section.
      }
      continue;
    }

    //assume normal line.
    string var,value;
    char *pos=::find(&buf[0],&buf[len],'='); //get the first '='
    if(pos==&buf[len]) {  //no '=' found, assume line all variable
      var=removeBlanks(string(&buf[0],len));
      value="";
    } else { //'=' found at pos.
      var=removeBlanks(string(&buf[0],pos-&buf[0])); //don't include '='.
      value=removeBlanks(string(pos+1,&buf[len]-(pos+1))); //no '='
    }

    if(var[0]=='-') { //minus here means 'remove variable'.
      //var.erase(0,1); //remove first character (the '-'). //old lib
      var.remove(0,1);
      theMap[section].erase(var); //erase variable from this sec.
    } else { //set variable as usual.
      theMap[section][var]=value;         //set the value.
    }
  }
}

void configFile::clear(void) {
  //erase from the beginning to the end.
  theMap.erase(theMap.begin(),theMap.end());
}

void configFile::reload(void) {
  clear();
  load();
}

int configFile::remove(const string& sec) {
  writeToCfg("[-" + sec + "]\n");
  return theMap.erase(sec);
}

int configFile::remove(const string& sec,const string& var) {
  writeToCfg("[" + sec + "]\n" + 
	     "-" + var + "\n");
  return theMap[sec].erase(var);
}
void configFile::set(const string &sec,const string& var,const string& val) {
  theMap[sec][var]=val;
  writeToCfg("[" + sec + "]\n" +
	     var + "=" + val + "\n");
}
string configFile::get(const string& sec, const string& var) const {
  return ((stringMapMap )theMap)[sec][var];
}

void configFile::writeToCfg(const string what) {
  ofstream cfgFile(fileName.c_str(),ofstream::app); //append
  cfgFile << what;
}

cfgIterator configFile::begin(void) { 
  return cfgIterator(theMap.begin());  
}
cfgIterator configFile::end(void) { 
  return cfgIterator(theMap.end());    
}
cfgIterator configFile::find(const string& k) { 
  return cfgIterator(theMap.find(k));  
}
bool configFile::exists(const string& k) { 
  return theMap.find(k)!=theMap.end(); 
}
stringMapMap::size_type configFile::size(void) {
  return theMap.size();
}


configSection::configSection(const string& n,configFile& c) : 
  name(n), cfg(c) { }
int configSection::remove(void) {
  return cfg.remove(name);
}
int configSection::remove(const string& var) {
  return cfg.remove(name,var); 
}
void configSection::set(const string& var,const string& val) { 
  cfg.set(name,var,val); 
}
string configSection::get(const string& var) const {
  return cfg.get(name,var); 
}

//FIXME: compiler trouble.
//typedConfigFile<class T>::typedConfigFile(string fName) : 
//  configFile(fName) { }

configVariable::configVariable(const string& n,configSection& s) : 
  name(n), section(s) { }
int configVariable::remove(void) {
  return section.remove(name); 
}

//FIXME: seems I can't outline these. Compiler trouble with
//template instantiations..
//configArray<class T>::configArray(const string& p,configSection& s) :
//  prefix(p), section(s) { }
//cfgNumVar<class T>::cfgNumVar(const string& n,configSection& s) : 
//  configVariable(n,s) { }

cfgStrVar::cfgStrVar(const string& n,configSection& s) : 
  configVariable(n,s) { }
void cfgStrVar::operator=(const string& s) {
  section.set(name,s); 
}
cfgStrVar::operator string() const {
  return section.get(name);
}

databaseSection::databaseSection(const string& n,configFile& c) : 
  configSection(n,c), ID("ID",*this), dir("directory",*this), 
  algs("alg_",*this) { }

userSection::userSection(const string& n,configFile& c) : 
  configSection(n,c), authLevel("authLevel",*this), 
  changePass("changePass",*this), password("password",*this), 
  dbAuth("DB_",*this) { }

algorithmSection::algorithmSection(const string& n,configFile& c) : 
  configSection(n,c), 
  algClass("class",*this), paramWin("paramWin",*this), 
  active("active",*this), replacedWithID("replacedWith",*this) { }

parameterSection::parameterSection(const string& n,configFile& c) : 
  configSection(n,c), http("http",*this), info("info",*this), 
  maxclients("maxclients",*this), timeout("timeout",*this) { }

algorithmSection algoConfigFile::operator[](const algorithmID_t k) {
  return algorithmSection(itos((int)k),*this);
}
cfgAlgIterator algoConfigFile::begin(void) { 
  return cfgAlgIterator(theMap.begin());  
}
cfgAlgIterator algoConfigFile::end(void) { 
  return cfgAlgIterator(theMap.end());    
}
cfgAlgIterator algoConfigFile::find(algorithmID_t k) { 
  return cfgAlgIterator(theMap.find(itos((int)k)));  
}
bool algoConfigFile::exists(algorithmID_t k) { 
  return theMap.find(itos((int)k))!=theMap.end(); 
}


istream& skipSpace(istream& is) {   //removes space from an istream.
  char c;

  while(is.get(c)) {
    if(!isspace(c)) {
      is.putback(c);
      break;
    }
  }
  return is;
}

string removeBlanks(string s) {   //returns a subs of s sans surronding spaces.
  int first, last;

  for(first=0;first<(int)s.length() && isspace(s[first]);++first);
  for(last=s.length()-1;last>=0 && isspace(s[last]);--last);
  if(last<first) { // none found!
    return "";
  }
  return s.substr(first,last-first+1);
}

//FIXME: seems my g++ is unable to generate all necessary templates--just
//my FUCKING luck. I'll do a few explicit instatiations here. These will
//make cfgread.o HUGE, but I guess that can't very well be helped.

template class typedConfigFile<databaseSection>; //instatiate whole classes.
template class typedConfigFile<userSection>;
template class typedConfigFile<algorithmSection>; //this not necessary
template class typedConfigFile<parameterSection>;

template class cfgNumVar<databaseID_t>;
template class cfgNumVar<algorithmID_t>;
template class cfgNumVar<DatabaseAuth>;
template class cfgNumVar<ServerAuth>;
template class cfgNumVar<unsigned int>;
template class cfgNumVar<bool>;
template class cfgNumVar<int>;

template class configArray<cfgNumVar<bool> >;
template class configArray<cfgNumVar<DatabaseAuth> >;







