#include<newdb.h>
#include<cfgread.h>
#include<string>
#include<gdbm.h>
#include<limits.h>	//for pathmax
#include<iostream.h>
#include<map.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<unistd.h>

static const int LOCKATTEMPTS=16;

bool dbHandler::lock(lockType type) {
  if(lockStatus==type || lockStatus==writeLock) //suffices
    return true;

  unlock(); //need to upgrade.

  if(type==readLock) {
    for(int i=0;i<LOCKATTEMPTS;i++) {
      gdbm_errno=GDBM_NO_ERROR;
      dbFile=gdbm_open((char *)fileName.c_str(),0,GDBM_READER,0600,NULL);
      if(gdbm_errno==GDBM_NO_ERROR) {
        lockStatus=type;
        return true;
      }
      cout << "[DB]: failed to read lock, attempt " << i+1;
      sleep(1); //this is very bad..
    }
  } else if(type==writeLock) {
    for(int i=0;i<LOCKATTEMPTS;i++) {
      gdbm_errno=GDBM_NO_ERROR;
      dbFile=gdbm_open((char *)fileName.c_str(),0,GDBM_WRCREAT,0600,NULL);
      if(gdbm_errno==GDBM_NO_ERROR) {
        lockStatus=type;
        return true;
      }
      cout << "[DB]: failed to write lock, attempt " << i+1;
      sleep(1); //very bad indeed.
    }
  } else { //not a read nor a write lock!
    cout << "[DB]: unknown lock type!" << endl;
    return false;
  }
  cerr << "failure to aquire database lock!" << endl;
  return false;
}
void dbHandler::unlock(void) {
  if(lockStatus==noLock)
    return;
  gdbm_close(dbFile);
  lockStatus=noLock;
}

dbHandler::dbHandler(const string fName) : fileName(fName) {
  lockStatus=noLock;
}
dbHandler::~dbHandler(void) { }

datum dbHandler::fetch(datum k) {
  datum d;
  d.dptr=0;

  if(!lock(readLock))
    return d;

  d=gdbm_fetch(dbFile,k);
  unlock();
  return d;
}
bool dbHandler::store(datum k, datum d) {
  if(!lock(writeLock))
    return false;
  bool stat=!gdbm_store(dbFile,k,d,GDBM_REPLACE);
  unlock();
  return stat;
}
bool dbHandler::remove(datum k) { 
  if(!lock(writeLock))
    return false;
  bool stat=!gdbm_delete(dbFile,k);
  unlock();
  return stat;
}
datum dbHandler::firstKey(void) { 
  datum d;
  d.dptr=0;

  if(!lock(readLock))
    return d;

  d=gdbm_firstkey(dbFile);
  unlock();
  return d;
}
datum dbHandler::nextKey(datum k) { 
  datum d;
  d.dptr=0;

  if(!lock(readLock))
    return d;

  d=gdbm_nextkey(dbFile,k);
  unlock();
  return d;
}

bool imageDB::saveImage(string fileName,image img) {
  string pName=imagePathName(fileName);

  cout << "saving image to " << pName << endl;

  int file=creat(pName.c_str(), S_IRUSR | S_IWUSR);

  if(file==-1) {
    cerr << "[imgDB] saveImage : failed to creat file!" << endl;
    return false;
  }

  int i,left=img.size;
  while(left) { 
    i=write(file,(void*)img.img,left);
    if(i==-1) { 
      cerr << "[imgDB] saveImage : write failure!" << endl;
      close(file);
      return false;
    }
    left-=i;
  }
  close(file);
  return true;
}

image imageDB::loadImage(string fileName) {
  string pName=imagePathName(fileName);
  int file=open(pName.c_str(),O_RDONLY);
  image img,failImg;

  failImg.img=NULL;	//to specify fail
     
  if(file==-1)
    return failImg;	//no file
  
  if((img.size=lseek(file,0,SEEK_END))==-1) {
    close(file);	//seek failed
    return failImg;
  }

  if(lseek(file,0,SEEK_SET)==-1) {
    close(file);	//seek failed
    return failImg;
  }
       
  if((img.img=(char *)malloc(img.size))==0) {
    close(file);	//no mem
    return failImg;
  }

  char *pos=(char *)img.img;
  int i,left=img.size;
  while(left) {
    i=read(file,pos,left);
    if(i==-1 || i==0) {	
      close(file);	//fail or premature eof
      free(img.img);
      return failImg;
    }
    left-=i;
    pos+=i;
  }
  return img; 	//success
}

picture *imageDB::fetchPic(pictureID_t ID) {
  picKey key(ID);
  datum d=picDB.fetch(key);
  return (picture *)d.dptr;
}

imageDB::imageDB(string dir) : picDB(dir + "/" + "picDB"), directory(dir) { 
  picKeyValid=sigKeyValid=false; //no fetches done yet.
}


bool imageDB::storePic(pictureID_t ID,const char *fileName, 
		       const char *paramStr1, const char *paramStr2,
		       unsigned int thumbSize, const char *thumbNail) {
  picture *storePic=(picture *)new char[sizeof(picture)+thumbSize];
  
  strncpy(storePic->fileName,fileName,_POSIX_PATH_MAX);
  storePic->fileName[_POSIX_PATH_MAX]=0;
  strncpy(storePic->paramStr1,paramStr1,PARAMSTRING_MAX);
  storePic->paramStr1[PARAMSTRING_MAX]=0;
  strncpy(storePic->paramStr2,paramStr2,PARAMSTRING_MAX);
  storePic->paramStr2[PARAMSTRING_MAX]=0;
  storePic->thumbSize=thumbSize;
  memcpy(storePic->thumbNail,thumbNail,thumbSize);

  picKey key(ID);
  datum d;

  d.dptr=(char *)storePic;
  d.dsize=sizeof(picture)+storePic->thumbSize;
  bool stat=picDB.store(key,d);

  delete[] storePic;
  return stat;
}

bool imageDB::removePic(pictureID_t ID) {
  picKey key(ID);
  return picDB.remove(key);
}
   
pictureID_t imageDB::firstPicKey(void) { 
  picKey key=picDB.firstKey();
  picKeyValid=key.valid;
  return key;
}

pictureID_t imageDB::nextPicKey(pictureID_t oldKey) {
  picKey fetchKey(oldKey);
  picKey key=picDB.nextKey(fetchKey);
  picKeyValid=key.valid;
  return key;
}
   
signature *imageDB::fetchSig(algorithmID_t alg,pictureID_t pic) {
  picKey key(pic);

  if(sigDBs[alg]==NULL) //no such signature DB!
    return NULL;
  datum d=sigDBs[alg]->fetch(key);
  return (signature *)d.dptr;
}

bool imageDB::storeSig(algorithmID_t alg,pictureID_t pic,
		       unsigned int sigSize,const char *sig) {
  if(sigDBs[alg]==NULL) 
    return false;  //no such DB!

  signature *storeSig=(signature *)new char[sizeof(signature)+sigSize];  
  storeSig->sigSize=sigSize;
  memcpy(storeSig->sig,sig,sigSize);

  datum d;
  picKey key(pic);

  d.dptr=(char *)storeSig;
  d.dsize=sizeof(signature)+storeSig->sigSize;
  bool stat=sigDBs[alg]->store(key,d);

  delete[] storeSig;
  return stat;
}

bool imageDB::removeSig(algorithmID_t alg,pictureID_t pic) {
  picKey key(pic);

  if(sigDBs[alg]==NULL) 
    return false;  //no such DB!
  return sigDBs[alg]->remove(key);
}

pictureID_t imageDB::firstSigKey(algorithmID_t alg) {
  if(sigDBs[alg]==NULL) 
    return 0;  //no such DB, 0 should indicate no pic.
  picKey key=sigDBs[alg]->firstKey();
  sigKeyValid=key.valid;
  return key;
}

pictureID_t imageDB::nextSigKey(algorithmID_t alg,pictureID_t oldKey) {
  picKey fetchKey(oldKey);
 
  if(sigDBs[alg]==NULL) 
    return 0;  //no such DB, 0 should indicate no pic.
  picKey key=sigDBs[alg]->nextKey(fetchKey);
  sigKeyValid=key.valid;
  return key;
}

string imageDB::imagePathName(string fileName) {
  return directory + "/images/" + fileName;
}

string imageDB::sigDBPathName(algorithmID_t alg) {
   return directory + "/sigs" + itos((int)alg);
} 

bool imageDB::addSigDB(algorithmID_t alg) {
  const string pName=sigDBPathName(alg);
  
  if((sigDBs.find(alg)!=sigDBs.end()) && sigDBs[alg]) 
    return false;   //already there, can't add it.
  sigDBs[alg]=new dbHandler(pName); //add.
  return true;
}

bool imageDB::removeSigDB(algorithmID_t alg) {
  if((sigDBs.find(alg)==sigDBs.end()) || !sigDBs[alg]) 
    return false; //not there, can't remove.
  delete sigDBs[alg];
  sigDBs.erase(alg); //get rid of the entry, too.
  return true;
}

bool newPICDB::addImgDB(databaseID_t DBID,const string& dir) {
  cout << "adding db " << DBID << endl;

  if((imgDBs.find(DBID)!=imgDBs.end()) && imgDBs[DBID]) {
    cerr << "image db already present!" << endl;
    return false; //already there, can't add it.
  }

  setNewID(DBID,1);  //set picture IDs in this DB to start from 1.

  imageDB *imgDB=new imageDB(dir);
  imgDBs[DBID]=imgDB; //add.

  string DBName="";
  for(cfgIterator i=DBs.begin();i!=DBs.end();++i) { //find db name..
    if(DBs[i.name()].ID==DBID) {
      DBName=i.name();
      break;
    }
  }

  if(DBName=="") {
    cerr << "database " << (int)DBID << " not found!";
    return true; //at least we added the imgDB..
  }

  for(cfgAlgIterator ai=algs.begin();ai!=algs.end();++ai) {
    algorithmID_t algID=ai.ID();

    if(DBs[DBName].algs[algID]) { //this algorithm active in this DB
      cout << "adding sig db for alg " << (int)algID << 
	" to db " << (int)DBID << endl;
      if(!imgDB->addSigDB(algID)) {
	cerr << "failed to add sig db for alg " << (int)algID <<
	  " to db " << (int)DBID << endl;
      }
    }
  }
  return true;
}

bool newPICDB::removeImgDB(databaseID_t ID) {
  if((imgDBs.find(ID)==imgDBs.end()) || !imgDBs[ID]) 
    return false; //not there, can't remove.
  delete imgDBs[ID];
  imgDBs.erase(ID); //get rid of the entry, too.

  IDDBKey key(ID);
  keyDB.remove(key); //remove this DB's entry from the ID DB.
  //FIXME: add error handling here.
  
  return true;
}

imageDB *newPICDB::getImgDB(databaseID_t ID) {
  imageDBMap::iterator imgDBi=imgDBs.find(ID);
  if(imgDBi==imgDBs.end()) //none found
    return NULL;
  return (*imgDBi).second;
}

string newPICDB::DBPathName(string dbName) {
  return "database/" + dbName;
}

newPICDB::newPICDB(void) : keyDB("database/keyDB"),
  DBs("database/DBs.cfg"), users("database/users.cfg"), 
  algs("database/algs.cfg"), params("database/params.cfg") { 

  //check that the key database holds suitable values.
  if(getNewID(0)==0) //assume the server DB ID has not been set
    setNewID(0,1);   //set it to start from 1.

  for(cfgIterator i=DBs.begin();i!=DBs.end();++i) {
    databaseID_t DBID=DBs[i.name()].ID;

    if(!addImgDB(DBID,DBPathName(DBs[i.name()].dir))) {
      cerr << "failed to add DB " << (int)DBID << endl;
    }
  }
}

unsigned int newPICDB::getNewID(databaseID_t DB) {
  IDDBKey key(DB);
  IDDBVal newID=keyDB.fetch(key);

  if(!newID.valid) //fetch failure
    return 0; //no ID may be zero, so use it to indicate failure.

  setNewID(DB,newID+1); //increment the ID value.
  return newID;
}

void newPICDB::setNewID(databaseID_t DB,unsigned int value) {
  IDDBKey key(DB);
  IDDBVal newID(value);
  
  keyDB.store(key,newID); //attempt to set ID count.
  //FIXME: add error handling here.
}

algorithmID_t newPICDB::newAlgID(void) {
  return (algorithmID_t)getNewID(0); //0 indicates server DB.
}
databaseID_t newPICDB::newDBID(void) {
  return (databaseID_t)getNewID(0); //0 indicates server DB.
}
pictureID_t newPICDB::newPicID(databaseID_t DB) {
  return (pictureID_t)getNewID(DB);
}











