// **********************************************************************
//
//      User.cpp
//
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <iostream.h>
#include <PICDebug.h>
#include <PICNet.h>
#include <PICServer.h>

void PICServer::SendUsers(void) {
  userData_msg msg;

  msg.users.users_len=DB.users.size();
  msg.users.users_val=new UserRights[msg.users.users_len];

  unsigned int userc=0;
  for(cfgIterator ui=DB.users.begin();ui!=DB.users.end();++ui) {
    UserRights& usr=msg.users.users_val[userc++];
    const string& userName=ui.name();
    userSection userInfo=DB.users[userName];

    usr.name=strdup(userName.c_str());
    usr.authLevel=userInfo.authLevel;
    usr.changePW=userInfo.changePass;

    usr.DBs.DBs_len=DB.DBs.size();
    usr.DBs.DBs_val=new DBRights[usr.DBs.DBs_len];
    
    unsigned int DBc=0;
    for(cfgIterator DBi=DB.DBs.begin();DBi!=DB.DBs.end();++DBi) {
      DBRights& DBR=usr.DBs.DBs_val[DBc++];
      const string& DBName=DBi.name();
      databaseSection DBInfo=DB.DBs[DBName];

      DBR.ID=DBInfo.ID;
      DBR.auth=userInfo.dbAuth[DBName];
    }
  }
  xdrH->send(&msg);
  LOG_MSG("[SRV] user data message sent\n");
  //FIXME: leaks mem.
}

void PICServer::ChangeUserRights(changeRights_msg &msg) {
  DEBUG_MSG("[SRV] ChangeUserRights" << endl);

  for(unsigned int userc=0;userc<msg.users.users_len;++userc) {
    const UserRights &usr=msg.users.users_val[userc];
   
    DB.users[usr.name].authLevel=usr.authLevel;
    DB.users[usr.name].changePass=usr.changePW;
    for(unsigned int DBc=0;DBc<usr.DBs.DBs_len;++DBc) {
      const DBRights& DBR=usr.DBs.DBs_val[DBc];

      for(cfgIterator DBi=DB.DBs.begin();DBi!=DB.DBs.end();++DBi) {
	if(DB.DBs[DBi.name()].ID==DBR.ID); {
	  DB.users[usr.name].dbAuth[DBi.name()]=DBR.auth;
	}
      }
    }
  }
  //FIXME: get rid of this CRAP!
  SendComplete();
}

void PICServer::AddUser(addUser_msg &msg) {
  DEBUG_MSG("[SRV] adding user '" << msg.name << "' with password '" << 
	    msg.passW << endl);

  //FIXME: instead of guessing, make this func check what the anon access
  //levels for the server and each DB are and give the new user the same.
  DB.users[msg.name].password=msg.passW;
  DB.users[msg.name].changePass=1; //allow by default
  DB.users[msg.name].authLevel=UserAccess;
  for(cfgIterator DBi=DB.DBs.begin();DBi!=DB.DBs.end();++DBi) {
    DB.users[msg.name].dbAuth[DBi.name()]=DB_ReadAccess;
  }

  //FIXME: this might be useful as a separate func ("sendUser")
  newUser_msg reply;
  UserRights &usr=reply.user;
  const string userName=msg.name;

  usr.name=(char *)userName.c_str();
  usr.authLevel=DB.users[userName].authLevel;
  usr.changePW=DB.users[userName].changePass;

  usr.DBs.DBs_len=DB.DBs.size();
  usr.DBs.DBs_val=new DBRights[usr.DBs.DBs_len];

  unsigned int DBc=0;
  for(cfgIterator DBi=DB.DBs.begin();DBi!=DB.DBs.end();++DBi) {
    DBRights &DBR=usr.DBs.DBs_val[DBc++];

    DBR.ID=DB.DBs[DBi.name()].ID;
    DBR.auth=DB.users[userName].dbAuth[DBi.name()];
  }
  xdrH->send(&reply);
  //FIXME: leaks mem.
}

void PICServer::RemoveUser(deleteUser_msg &msg) {
  DEBUG_MSG("[SRV] removing user '"<< msg.name <<"'\n");
  
  DB.users.remove(msg.name);
  SendComplete();
}

void PICServer::ChangePassword(changePW_msg &msg) {
  DEBUG_MSG("[SRV] ChangePassword for " << msg.name << " from " << 
	    msg.oldPW << " to " << msg.newPW);

  if (server_auth!=SuperUser) {
    DEBUG_MSG("[SRV] checking old password\n");

    //FIXME: compiler trouble with ambiguous !=.
    string PW=DB.users[msg.name].password;
    if(PW.compare(msg.oldPW)!=0) {
      DEBUG_MSG("[SRV] old password does not match\n");
      SendFail();
      return;
    }
  }
  DEBUG_MSG("[SRV] changing '"<< msg.name <<"'s password from '" << 
	    msg.oldPW << "' to ' " << msg.newPW << endl);

  DB.users[msg.name].password=msg.newPW;
  SendComplete();
}











