// **********************************************************************
//  This file contains miscellaneous routines of PicMain
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <mmain.hxx>
#include <pinteger.hxx>
#include <ptext.hxx>
#include <pstrstm.hxx>
#include <iostream.h>

// **********************************************************************

PICBOOL PicMain::calcAllSegments(PicPicture &image)
{
  return boolTrue;
}

// **********************************************************************

PICBOOL PicMain::calcAllSignatures(PicPicture &image)
{
	PicIterator iter;
	PicSearchAlgorithm *tmp;
	PicSignature *sig;

	image.signatures.listClean();
  
	globData.searchAlgs.listRewind();

	while((tmp=(PicSearchAlgorithm *)globData.searchAlgs.listNext()))
	{
		sig = tmp->computeSignature(image);
		if(!sig || PicError::isDetected())
		{
			cout << "[SRV] Calculation failure for " << tmp->algName() << "\n";
			return(boolFalse);
		}
		sig->creator_id=tmp->algID();
    	image.signatures.listAdd((void *)sig);
	}
	
	return boolTrue;
}

// **********************************************************************

PicImageFormat* PicMainSharedData::getImageFormat(PICULONG id)
{
  PicIterator iter;
  ITERATE(imageFormats, iter)
  {
    PicImageFormat &curr = AS(PicImageFormat, imageFormats.at(iter));
    if(id == curr.id)
      return &curr;
  }
  return NULLPTR;
}

// **********************************************************************

PicSearchAlgorithm* PicMainSharedData::getSearchAlg(PICULONG id)
{
	PicSearchAlgorithm *tmp;

	searchAlgs.listRewind();

	while((tmp=(PicSearchAlgorithm *)searchAlgs.listNext()))
	{
		if(tmp->algID()==id)
			return(tmp);
	}

	return(NULL);
	
}

// **********************************************************************

PicSegmentAlgorithm* PicMainSharedData::getSegmentAlg(BaseString name)
{
  PicIterator iter;
  ITERATE(segmentationAlgs, iter)
  {
    PicSegmentAlgorithm &curr = AS(PicSegmentAlgorithm, 
                                   segmentationAlgs.at(iter));
    if(name == curr.name)
      return &curr;
  }
  return NULLPTR;
}

PicSegmentAlgorithm* PicMainSharedData::getSegmentAlg(PICULONG id)
{
  PicIterator iter;
  ITERATE(segmentationAlgs, iter)
  {
    PicSegmentAlgorithm &curr = AS(PicSegmentAlgorithm, 
                                   segmentationAlgs.at(iter));
    if(id == curr.id)
      return &curr;
  }
  return NULLPTR;
}

// **********************************************************************

static void stdoutPrintErrors(void)
{
}

