// **********************************************************************
//  Implementation of the auxiliarity threads for PicCore
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <mmain.hxx>
#include <PicSignature.hxx>

#include <perrmgr1.hxx>
#include <pfilestm.hxx>
#include <pstrstm.hxx>
#include <ptext.hxx>
#include <pixmap.hxx>

// **********************************************************************

#define EXIT_ON_COMMAND()   if(checkCommand()) goto endThread;

// **********************************************************************

MAKE_PICVIRTCLASS(PicEnhancedThread, PicThread)

PicEnhancedThread::PicEnhancedThread(PicPriority priority)
  : PicThread(priority),
    command(go),
    name("Noname")
{

}


PICBOOL PicEnhancedThread::checkCommand()
{
  return boolTrue;
}

// **********************************************************************

MAKE_PICVIRTCLASS(PicQueryThread, PicEnhancedThread)

ERROR_MESSAGES(PicQueryThread)
PARAMSTR(noExampleImages, "Query: No query image")
PARAMSTR(noSearchAlg, "Query: No search algorithm selected")
PARAMSTR(noQueryAvailable, "No query results available")
END_PARAM_STRINGS

PicQueryThread::PicQueryThread(PicPriority priority)
  : PicEnhancedThread(priority), 
    exImages(NULLPTR),
    searchAlg(NULLPTR)
{ 
  name = "Query";
}

// **********************************************************************

PICTHREADRESULT PicQueryThread::threadFunction(PICTHREADARG arg)
{
  return 0;
}

// **********************************************************************

PICBOOL PicQueryThread::queryLoop(PicSignature *exSign,
                                  PicSignature *testSign)
{
  return boolTrue;
}

// **********************************************************************

PICUSHORT PicQueryThread::donePercentage()
{
  return 0;
}

// **********************************************************************

BaseString PicQueryThread::queryTime()
{
  return "";
}

// **********************************************************************
// **********************************************************************


MAKE_PICVIRTCLASS(PicMassInsertThread, PicEnhancedThread)

ERROR_MESSAGES(PicMassInsertThread)
PARAMSTR(noFileName, "Batch insertion: No file name specified")
PARAMSTR(fileNotExist, "Batch insertion: File @* not found")
END_PARAM_STRINGS

PicMassInsertThread::PicMassInsertThread(PicPriority priority)
  : PicEnhancedThread(priority),
    insertedPics(0), failedPics(0), notFound(0),
    inputStream(NULLPTR)
{
  name = "Batch insertion";
}

// **********************************************************************

PICBOOL PicMassInsertThread::doInits()
{
  return boolFalse;
}

// **********************************************************************

PICBOOL PicMassInsertThread::extractParams(BaseString line, 
                                           BaseString &fName, 
                                           PicPicture &newPic)
{
  return boolTrue;  
}

// **********************************************************************

PICBOOL PicMassInsertThread::printStats()
{
  return boolTrue;
}
// **********************************************************************

PICBOOL PicMassInsertThread::insertPic(PicPicture &newPic)
{
  return boolTrue;
}

// **********************************************************************

PICTHREADRESULT PicMassInsertThread::threadFunction(PICTHREADARG arg)
{
  return 0;
}

// **********************************************************************
// **********************************************************************

MAKE_PICVIRTCLASS(PicDbUpdateThread, PicEnhancedThread)

//  ERROR_MESSAGES(PicDbUpdateThread)
//  PARAMSTR(,)
//  END_PARAM_STRINGS

PicDbUpdateThread::PicDbUpdateThread(PicPriority priority)
  : PicEnhancedThread(priority)
{
  name = "Database update";
}

// **********************************************************************

PICBOOL PicDbUpdateThread::removeOldEntries()
{
  return boolTrue;
}

// **********************************************************************

PICBOOL PicDbUpdateThread::insertAlgNames()
{
  return boolTrue;
}

// **********************************************************************

PICBOOL PicDbUpdateThread::doSegments(PicPicture &pic)
{
  return boolTrue;
}

// **********************************************************************

PICBOOL PicDbUpdateThread::doSignatures(PicPicture &pic)
{
  return boolTrue;
}

// **********************************************************************

PICTHREADRESULT PicDbUpdateThread::threadFunction(PICTHREADARG arg)
{
  return 0;
}

