typedef unsigned int algorithmID_t; 
typedef unsigned int databaseID_t;
typedef unsigned int pictureID_t;

enum ServerAuth {
  NoAccess,
  UserAccess,
  SuperUser
};

enum DatabaseAuth {
  DB_NoAccess,
  DB_ThumbAccess,
  DB_ReadAccess,
  DB_WriteAccess
}; 

struct DBInfo {
  databaseID_t ID;
  DatabaseAuth authLevel;
  string name<>;
  algorithmID_t algs<>;  
};

struct AlgInfo {
  algorithmID_t ID;
  string name<>;
  string description<>;
  string algParamWin<>;
};

struct strStruct {	/* sorry, will fix. */
  string str<>;
};

struct namedValue {
  string name<>;
  string value<>;
};

struct DBRights {
  databaseID_t ID;
  DatabaseAuth auth;
};

struct UserRights {
  string name<>;
  ServerAuth authLevel;
  bool changePW;
  DBRights DBs<>;
};

struct thumbNail {
  opaque data<>;
  pictureID_t picID;
  string paramStr1<>;
  string paramStr2<>;
  unsigned int rank;
  unsigned int match;
};

enum ackValue {
  ack_OK,
  ack_unauthorised,
  ack_busy,
  ack_failed,
  ack_noQueryMatch,
  ack_timeout
};

enum queryType {
  queryByPicture,
  queryByID
};

/******************** START OF MESSAGES  ****************************/

struct ack_msg {
  ackValue value;    
};

struct init_msg {               /* message00 */
  ServerAuth authLevel;
  bool changePW;
  DBInfo DBs<>;
  AlgInfo algs<>;
  string serverAd<>;
  string serverHTTP<>;
};

struct hangup_msg {		/* message01 */
  unsigned int time;
}; 

struct query_msg {		/* message02 */
  databaseID_t DBID;
  algorithmID_t algID;
  strStruct params<>;
  unsigned int thumbMax;
  unsigned int thumbNum;
  unsigned int minMatch;
  databaseID_t qPicDB;
  pictureID_t qPicID;    
  queryType type;
};

struct stopQuery_msg {	/* message03 */
  int foo;
};

struct moreThumbs_msg {		/* message04 */
  unsigned int firstThumb;
  unsigned int thumbNum;
};

struct insertPic_msg {		/* message05 */
  string fileName<>;
  opaque picData<>;
  string paramStr1<>;
  string paramStr2<>;
  databaseID_t picDB;
  pictureID_t picID;  
};

struct getPic_msg {		/* message06 */
  databaseID_t picDB;
  pictureID_t picID;
};

struct deletePic_msg {		/* message07 */
  databaseID_t picDB;
  pictureID_t picID;
};

struct disconnect_msg { /* message08 */
  int foo;
};

struct modifyAlgs_msg {		/* message09 */
  string targetDB<>;
  algorithmID_t usedAlgs<>;
};

struct getUsers_msg {		/* message10 */
  int dummy;
};

struct changeRights_msg {	/* message11 */
  UserRights users<>;
};

struct addUser_msg {		/* message12 */
  string name<>;
  string passW<>;
};  

struct deleteUser_msg {		/* message13 */
  string name<>;
};

struct changePW_msg {		/* message14 */
  string name<>;
  string newPW<>;
  string oldPW<>;
};

struct createDB_msg {		/* message15 */
  string directory<>;
  string DBName<>;
  algorithmID_t algs<>;
};

/* messages 16&17 removed */

struct deleteDB_msg {		/* message18 */
  string DBName<>;
};

struct getServOpt_msg {		/* message19 */
  int dummy;
};

struct setServOpt_msg { 	/* message20 */
  namedValue opts<>;
};

struct progReport_msg {		/* message21 */
  int status;
};

struct queryResults_msg {	/* message22 */
  struct thumbNail nails<>;  
};

struct queryNoResults_msg {	/* message25 */
  int dummy;
};

struct picData_msg {		/* message27 */
  string fileName<>;
  opaque picData<>;  
  string paramStr1<>;
  string paramStr2<>;
  pictureID_t picID;
  unsigned int picWidth;
  unsigned int picHeight;
  unsigned int picFormat;
};

struct algProgress_msg {	/* message28 */
  bool opType;
  algorithmID_t algID;
};

struct userData_msg {		/* message29 */
  UserRights users<>;    
};

struct newUser_msg {		/* message30 */
  UserRights user;
};

struct newDB_msg {		/* message32 */
  DBInfo DB;  
};

struct servOpts_msg {		/* message33 */
  namedValue opts<>;
};

struct newPic_msg {		/* message34 */
  pictureID_t picID;
};

struct addAlg_msg {             /* message35 */
  string algLibPath<>;
  string paramWinPath<>;
  int algClass;
};

struct modifyAlg_msg {  	/* message36 */
  string algLibPath<>;
  string paramWinPath<>;
  algorithmID_t ID;
  int algClass;
  bool recalcSigs;
};

struct removeAlg_msg {  	/* message37 */
  algorithmID_t ID;
};



